/***************************************************************************
 *   Copyright (C) 2006 by Dave Parker                                     *
 *   drp@ezeedsl.co.uk                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "audioplayer.h"
#include "colour.h"
#include "font.h"
#include "music.h"
#include "surface.h"
#include <algorithm>

using namespace std;

AudioPlayer* AudioPlayer::instance = NULL;

AudioPlayer::AudioPlayer() : current_file ( 0 ), state ( STOPPED ), volume ( 64 )
{
}

/**
 * Display player
 * @param s - Destination Surface
 * @param f - Font
 * @param fg - Foreground Colour
 * @param bg - Background Colour
 * @param hl - Highlight Colour for selected option
 */
void AudioPlayer::Render ( Surface& s, const Font& f, const Colour &fg, const Colour &bg, const Colour &hl )
{
    int w,h;
    f.GetTextSize ( "", w, h );

    f.PutString ( s, 10, 20, fg, bg, "Play list" );

    for ( int i=current_file-3; i<current_file+5; i++ )
    {
        int pos = 3+i-current_file;
        Colour c ( bg );
        if ( i == current_file )
        {
            c = hl;
        }
        s.Rect ( 10, 40 - h/2 + pos * 20, 300, 20, c );
        if ( i >=0 && i<play_list.size() )
        {
            f.PutString ( s, 10, 40 + pos * 20 , fg, c,
                          basename ( play_list[i]->GetFile().c_str() ) );
        }
    }
}


bool AudioPlayer::AddFile ( string fn )
{
    Music *m = new Music ( fn );
    bool valid = m->IsValid();
    if ( valid )
    {
        play_list.insert ( play_list.end(), m );
    }

    return valid;
}

void AudioPlayer::Start()
{
    if ( !play_list.empty() )
    {
        state = PLAYING;
        Play ( play_list[current_file] );
    }
}

void AudioPlayer::Stop()
{
    state = STOPPED;
    if ( instance != NULL )
    {
        Mix_HaltMusic();
    }
}

void AudioPlayer::ToggleState()
{
    if ( state == PLAYING )
    {
        Stop();
    }
    else
    {
        Start();
    }

}

void AudioPlayer::Play ( Music* music )
{
    Mix_HookMusicFinished ( SMusicFinished );
    if ( music->Play() )
    {
        music->SetVolume ( volume );
        instance = this;
    }
    else
    {
        MusicFinished();
    }
}

void AudioPlayer::MusicFinished()
{
    instance = NULL;
    if ( !play_list.empty() )
    {
        if ( state == PLAYING )
        {
            // Start next file playing
            current_file = ( current_file + 1 ) % play_list.size();
            Start();
        }
    }
}

void AudioPlayer::SMusicFinished()
{
    if ( instance != NULL )
    {
        instance->MusicFinished();
    }
}



void AudioPlayer::NextFile()
{
    if ( play_list.empty() ) return;

    PlayerState old_state = state;
    Stop();
    current_file = ( current_file + 1 ) % play_list.size();
    if ( old_state == PLAYING )
    {
        // Was playing so restart
        Start();
    }
}


void AudioPlayer::PreviousFile()
{
    if ( play_list.empty() ) return;

    PlayerState old_state = state;
    Stop();
    current_file = ( current_file + play_list.size() - 1 ) % play_list.size();

    if ( old_state == PLAYING )
    {
        // Was playing so restart
        Start();
    }
}


void AudioPlayer::VolumeUp()
{
    if ( !play_list.empty() )
    {
        play_list[current_file]->VolumeUp();
        volume = play_list[current_file]->GetVolume();
    }
}


void AudioPlayer::VolumeDown()
{
    if ( !play_list.empty() )
    {
        play_list[current_file]->VolumeDown();
        volume = play_list[current_file]->GetVolume();
    }
}


void AudioPlayer::SetVolume ( int v )
{
    volume = v;
}


int AudioPlayer::GetVolume()
{
    return volume;
}

void AudioPlayer::RemoveFile()
{
    if ( play_list.empty() ) return;

    PlayerState old_state = state;
    Stop();

    vector<Music*>::iterator it = find ( play_list.begin(), play_list.end(),
                                         play_list[current_file] );
    if ( it != play_list.end() )
    {
        delete *it;
        play_list.erase ( it );
        if ( !play_list.empty() )
        {
            current_file %= play_list.size();
        }
    }

    if ( old_state == PLAYING )
    {
        // Was playing so restart
        Start();
    }
}


